#import "manualView.h"

@implementation manualView

- (id)initWithFrame:(NSRect)frameRect
{
	if ((self = [super initWithFrame:frameRect]) != nil) {
		imageToDisplay = [[NSImage alloc] initWithSize: NSMakeSize(640, 480)];
		[imageToDisplay retain];
		[[NSColor whiteColor] set];
		currentPicture = 0;
		// Add initialization code here
	}
	return self;
}

- (void)nextImage: (NSImage *)next
{
	[imageToDisplay setSize :[next size]];
	[imageToDisplay lockFocus];
	[next compositeToPoint: NSMakePoint(0,0) operation:NSCompositeSourceOver];
	[imageToDisplay unlockFocus];
	[self setFrame :NSMakeRect(0, 0, [imageToDisplay size].width, [imageToDisplay size].height)];
}

- (void)doManualWay: (NSString *)imagePath :(NSString *)returnPath :(NSArray *)actualImages: (BOOL) completeString :(NSString *)copyright :(BOOL)dropped :(float)transparency :(int)resize_X :(int)resize_Y :(int)resize_Percent;
{
	NSImage *sourceImage;
	
	resizeX = resize_X;
	resizeY = resize_Y;
	resizePercent = resize_Percent;
	
	imageArray = [NSArray arrayWithArray :actualImages];
	[imageArray retain];
	copyrightImage = [[NSImage alloc] initWithContentsOfFile: copyright];
	[copyrightImage retain];
	if (!completeString){
		path = [NSString stringWithString :imagePath];
		[path retain];
	} else {
		path = @"none";	
		[path retain];
	}
	savePath = [NSString stringWithString :returnPath];
	[savePath retain];
	droppedOnEmpty = dropped;
	trans = transparency;
//	NSLog ([imageArray objectAtIndex :currentPicture]);
	
	
	if (!completeString){
		sourceImage = [[NSImage alloc] initByReferencingFile:[NSString stringWithFormat: @"%@/%@", path, [imageArray objectAtIndex :currentPicture]]]; // wish I knew how to load a new picture without doing the initWith thing. Does this leak?
	}else {
		sourceImage = [[NSImage alloc] initByReferencingFile:[imageArray objectAtIndex :currentPicture]]; // wish I knew how to load a new picture without doing the initWith thing. Does this leak?
	}		
	
	

	while ( ![sourceImage isValid] ){
		currentPicture++;
		[sourceImage release];
//		NSLog ([imageArray objectAtIndex :currentPicture]);
		sourceImage = [[NSImage alloc] initByReferencingFile:[NSString stringWithFormat: @"%@/%@", path, [imageArray objectAtIndex :currentPicture]]]; // wish I knew how to load a new picture without doing the initWith thing. Does this leak?
	}
	[sourceImage setScalesWhenResized :YES];
	if ( resizeX != 0 | resizeY != 0 | resizePercent != 0 ){
		[sourceImage lockFocus];
		if ( resizePercent == 0 )
			[sourceImage setSize :NSMakeSize(resizeX, resizeY)];
		else
			[sourceImage setSize :NSMakeSize([sourceImage size].width * (resizePercent * .01), [sourceImage size].height * (resizePercent * .01))];
		[sourceImage unlockFocus];
	}	 
	
	[imageToDisplay setSize :[sourceImage size]];
	[imageToDisplay lockFocus];
	[sourceImage compositeToPoint: NSMakePoint(0,0) operation:NSCompositeSourceOver];
	[imageToDisplay unlockFocus];
	[self display:501  :640 :NO];
	
}

- (void)display:(float)windowWidth :(float)windowHeight :(BOOL)scale
{
	if ( scale ){
		[self setFrame :NSMakeRect(0, 0, windowWidth, windowHeight)];
		if ( [imageToDisplay size].width > windowWidth || [imageToDisplay size].height > windowHeight - 37 ){ // here's my cute little scale code. Basically it checks which side is bigger

			[imageToDisplay setScalesWhenResized :YES];

			if ( ((windowHeight  - 37) / [imageToDisplay size].height ) * [imageToDisplay size].width < windowWidth && ((windowHeight  - 37) / [imageToDisplay size].height ) * [imageToDisplay size].height < windowHeight){
				//			NSLog (@"scale 1");			
				[imageToDisplay setSize :NSMakeSize([imageToDisplay size].width * ( (windowHeight  - 37) / [imageToDisplay size].height ), [imageToDisplay size].height * ( (windowHeight  - 37) / [imageToDisplay size].height ) ) ];
			} else {
				//			NSLog (@"scale 2");			
				[imageToDisplay setSize :NSMakeSize([imageToDisplay size].width * ( windowWidth / [imageToDisplay size].width ), ( [imageToDisplay size].height - 37) * ( windowWidth / [imageToDisplay size].width ) )  ];
			}
		}
	} else
		[self setFrame :NSMakeRect(0, 0, [imageToDisplay size].width, [imageToDisplay size].height)];
//	NSLog(@"%f, %f",  [imageToDisplay size].width, [imageToDisplay size].height);
	[self lockFocus];
	[[NSColor whiteColor] set];
	NSRectFill ( NSMakeRect(0, 0, 640, 501) );
	[imageToDisplay compositeToPoint: NSMakePoint(0,0) operation:NSCompositeSourceOver];
	[self unlockFocus];
	[[self window] flushWindow];
};


- (void)drawRect:(NSRect)rect
{
	// pick out the next picture. if it's too big, scale it
	// If it's not in manual mode, put the CW over it, display it
	// if it is, your job is done until mouseup
	[[NSColor whiteColor] set];
	NSRectFill (rect);
	[imageToDisplay compositeToPoint: NSMakePoint(0,0) operation:NSCompositeSourceOver];

};

- (void)mouseDown:(NSEvent *)theEvent
{
	// put the CW over the picture
	
	NSPoint mousePoint = [self convertPoint: [theEvent locationInWindow] fromView:nil];
	if ( mousePoint.x < [imageToDisplay size].width && mousePoint.y < [imageToDisplay size].height ){
		[[NSColor whiteColor] set];
		[self lockFocus];
		NSRectFill ( NSMakeRect(0, 0, [imageToDisplay size].width, [imageToDisplay size].height) );
		[imageToDisplay compositeToPoint: NSMakePoint(0,0) operation:NSCompositeSourceOver];
		[copyrightImage compositeToPoint: mousePoint operation:NSCompositeSourceOver fraction:trans];
		[self unlockFocus];
		[[self window] flushWindow];
	} else
		NSBeep();
};

- (void)mouseUp:(NSEvent *)theEvent
{
	// get ready for the next one
	NSAutoreleasePool *pool; //Also, if you need to write a loop that creates many temporary objects, you should create an autorelease pool 
							 // in the loop to prevent too long a delay in the disposal of those objects.
							 // that's what was happening- the pictures were hanging around too long in memory even though they had been "released". So this gets rid
							 // of them right away. At the cost of a little time for garbage collection. Better that then chewing through all your memory though!
	
	pool = [[NSAutoreleasePool alloc] init];
	NSData              *colorSyncProfile = nil;

    NSImage *sourceImage;
	
	if (![path isEqual: @"none"])
		sourceImage = [[NSImage alloc] initByReferencingFile:[NSString stringWithFormat: @"%@/%@", path, [imageArray objectAtIndex :currentPicture]]]; // wish I knew how to load a new picture without doing the initWith thing. Does this leak?
	else
		sourceImage = [[NSImage alloc] initByReferencingFile:[imageArray objectAtIndex :currentPicture]]; // wish I knew how to load a new picture without doing the initWith thing. Does this leak?

	[sourceImage setScalesWhenResized :YES];
	if ( resizeX != 0 | resizeY != 0 | resizePercent != 0 ){
		[sourceImage lockFocus];
		if ( resizePercent == 0 )
			[sourceImage setSize :NSMakeSize(resizeX, resizeY)];
		else
			[sourceImage setSize :NSMakeSize([sourceImage size].width * (resizePercent * .01), [sourceImage size].height * (resizePercent * .01))];
		[sourceImage unlockFocus];
	}	 

	NSPoint mousePoint = [self convertPoint: [theEvent locationInWindow] fromView:nil];
	
	if ( mousePoint.x < [imageToDisplay size].width && mousePoint.y < [imageToDisplay size].height ){
		 
		NSBitmapImageRep *imageRep;
		NSSize imageSize;
		NSString *type;
		NSRect rect;
		NSBitmapImageFileType typeCode = -99;
		double compression = 0.0;
		NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

		compression = [defaults floatForKey :@"compression"];
		
		if (compression == 0){
			compression = .9;
			[defaults setInteger:compression forKey: @"compression"];
		}
		
		 type = [[[imageArray objectAtIndex :currentPicture] pathExtension] lowercaseString]; // get a dictionary of values from the file manager about this file and get the type from them
		 
		 if ( [type isEqualTo: @"pict"] || [type isEqualTo: @"pct"] || [type isEqualTo: @"PICT"] || [type isEqualTo: @"PCT"]){
			 NSRunAlertPanel(@"My Apologies",@"I regret to say I can not not process the PICT format at this time.\nThis is because I can't find its dimentions.\nSorry",@"OK",NULL,NULL);
		 }
		 
		 if ( [type isEqualTo: @"jpg"] || [type isEqualTo: @"jpeg"] || [type isEqualTo: @"JPG"] || [type isEqualTo: @"JPEG"] ){
			 typeCode = NSJPEGFileType;
		 }
		 if ( [type isEqualTo: @"gif"] || [type isEqualTo: @"GIF"])
		 typeCode = NSGIFFileType;
		 if ( [type isEqualTo: @"png"] || [type isEqualTo: @"PNG"])
		 typeCode = NSPNGFileType;
		 if ( [type isEqualTo: @"bmp"] || [type isEqualTo: @"BMP"])
		 typeCode = NSBMPFileType;
			if ( [type isEqualTo: @"tiff"]  || [type isEqualTo: @"TIF"] || [type isEqualTo: @"TIFF"] || [type isEqualTo: @"tif"])
		 typeCode = NSTIFFFileType;
		 
		 if ( typeCode == -99 ) { // it didn't change. I don't know what to do with it.
								  //                NSLog (@"Code didn't change on %@", type );
			 return;
		 }
 		 NSBitmapImageRep *TIFFRep = [[NSBitmapImageRep alloc] initWithData:[sourceImage TIFFRepresentation]];
		 colorSyncProfile = [[TIFFRep valueForProperty:NSImageColorSyncProfileData] copy];
		 imageSize = NSMakeSize([TIFFRep pixelsWide], [TIFFRep pixelsHigh]);
		 
		 [sourceImage setSize :NSMakeSize ([TIFFRep pixelsWide], [TIFFRep pixelsHigh])];
		 [sourceImage lockFocus];
		 [copyrightImage dissolveToPoint:NSMakePoint(mousePoint.x , mousePoint.y ) fraction:trans];
		 
		 // now, put the image in the right place relating to where they have chosen
		 
		 rect = NSMakeRect(0, 0, [sourceImage size].width, [sourceImage size].height);
		 imageRep = [[NSBitmapImageRep alloc] initWithFocusedViewRect:rect];
		 [sourceImage unlockFocus];
		 if (![imageRep isKindOfClass:[NSBitmapImageRep class]]) {
			 NSRunAlertPanel( @"My Apologies",  @"There was something wrong with that image", @"Ok", nil, nil );
			 [[self window] close]; // can't just leave them hangin in space, can I?
			 return;
		 }
		 [imageRep setSize :NSMakeSize([TIFFRep size].width, [TIFFRep size].height)]; // here's the line that puts it back
																						  // to the original dpi. If it's 72, it just gets put back the way it was
		 [imageRep setProperty:NSImageColorSyncProfileData withValue:colorSyncProfile];
		 [[imageRep representationUsingType:typeCode properties:[NSDictionary dictionaryWithObject:[NSNumber numberWithFloat:compression] forKey:NSImageCompressionFactor]] writeToFile:[NSString stringWithFormat :@"%@%@", savePath, [[imageArray objectAtIndex :currentPicture] lastPathComponent]] atomically:YES];
		 
		if ( currentPicture + 1 < [imageArray count] ){
			currentPicture++;
			[sourceImage release]; // turns out it was this (missing) that caused my leak. I was making a new sourceImage but didn't get rid of 
			// my old source image. The TIFFRep thing didn't help, either, actually. But now it doesn't loose memory anyplace!
			if (![path isEqual: @"none"])
				sourceImage = [[NSImage alloc] initByReferencingFile:[NSString stringWithFormat: @"%@/%@", path, [imageArray objectAtIndex :currentPicture]]]; // wish I knew how to load a new picture without doing the initWith thing. Does this leak?
			else
				sourceImage = [[NSImage alloc] initByReferencingFile:[imageArray objectAtIndex :currentPicture]]; // wish I knew how to load a new picture without doing the initWith thing. Does this leak?
			
			type = [[[imageArray objectAtIndex :currentPicture] pathExtension] lowercaseString]; // get a dictionary of values from the file manager about this file and get the type from them
			
			while ( ![sourceImage isValid] && currentPicture < [imageArray count] || ([type isEqualTo: @"pict"] || [type isEqualTo: @"pct"] || [type isEqualTo: @"PICT"] || [type isEqualTo: @"PCT"]) ){
				currentPicture++;
				[sourceImage release];
				if (![path isEqual: @"none"])
					sourceImage = [[NSImage alloc] initByReferencingFile:[NSString stringWithFormat: @"%@/%@", path, [imageArray objectAtIndex :currentPicture]]]; // wish I knew how to load a new picture without doing the initWith thing. Does this leak?
				else
					sourceImage = [[NSImage alloc] initByReferencingFile:[imageArray objectAtIndex :currentPicture]]; // wish I knew how to load a new picture without doing the initWith thing. Does this leak?
				
				type = [[[imageArray objectAtIndex :currentPicture] pathExtension] lowercaseString]; // get a dictionary of values from the file manager about this file and get the type from them
			}
			
// I think it's this TIFFRep that causing memory loss while the program runs in manual mode. It doesn't in the other mode because I only use
// TIFFRep once and then chuck it. Here I replace what WAS in TIFFRep with the next image, so everything in the old TIFFRep is still hanging
// around. It gets cleaned up when the program finishes, but it still should be taken care of as memory just spirals up as the thing runs
			NSBitmapImageRep *TIFFRep2 = [[NSBitmapImageRep alloc] initWithData:[sourceImage TIFFRepresentation]];
			imageSize = NSMakeSize([TIFFRep2 pixelsWide], [TIFFRep2 pixelsHigh]);
			[sourceImage setScalesWhenResized :YES];
			[sourceImage lockFocus];		
			
			if ( resizeX != 0 | resizeY != 0 | resizePercent != 0 ){

				if ( resizePercent == 0 )
					[sourceImage setSize :NSMakeSize(resizeX, resizeY)];
				else
					[sourceImage setSize :NSMakeSize([sourceImage size].width * (resizePercent * .01), [sourceImage size].height * (resizePercent * .01))];
				
			} else	 			
				[sourceImage setSize :NSMakeSize ([TIFFRep2 pixelsWide], [TIFFRep2 pixelsHigh])];
			[sourceImage unlockFocus];
			[imageToDisplay setSize :[sourceImage size]];
			[imageToDisplay lockFocus];
			[sourceImage compositeToPoint: NSMakePoint(0,0) operation:NSCompositeSourceOver];
			[imageToDisplay unlockFocus];
			[self display:501  :640 :NO];
			[TIFFRep release];
			[TIFFRep2 release];
			[sourceImage release];
			[imageRep release];
			[pool release];

		} else{
			[[self window] close];
			NSRunAlertPanel(@"Complete",@"I have completed that action.",@"OK",NULL,NULL);
			currentPicture = 0;
		}
	}
}

- (void)mouseDragged:(NSEvent *)event
{
	// update the region covered by the CW previously with what's there in the bg, and put the CW
	// in a new position under the mouse
	
	NSPoint mousePoint = [self convertPoint: [event locationInWindow] fromView:nil];
	if ( mousePoint.x < [imageToDisplay size].width && mousePoint.y < [imageToDisplay size].height ){
		[[NSColor whiteColor] set];
		[self lockFocus];
		NSRectFill ( NSMakeRect(0, 0, [imageToDisplay size].width, [imageToDisplay size].height) );
		[imageToDisplay compositeToPoint: NSMakePoint(0,0) operation:NSCompositeSourceOver];
		[copyrightImage compositeToPoint: mousePoint operation:NSCompositeSourceOver fraction:trans];
		[self unlockFocus];
		[[self window] flushWindow];
	} else
		NSBeep();
	
};

- (void) dealloc
{
	[imageToDisplay release];
	[copyrightImage release];
	
	[imageArray release];
	[path release];
	[savePath release];
	[super dealloc];
}
@end
